Exception: Rubikon::UnexpectedArgumentError

Inherits:
ArgumentError
  • Object
show all
Defined in:
lib/rubikon/errors.rb

Overview

Raised if an argument is passed, that does not match a validation rule

See Also:

  • HasArguments#check_args

Author:

  • Sebastian Staudt

Since:

  • 0.6.0

Instance Method Summary collapse

Constructor Details

#initialize(arg) ⇒ UnexpectedArgumentError

Creates a new error and stores the given argument value

Parameters:

  • arg (Symbol) —

    The given argument value

Since:

  • 0.6.0



89
90
91
# File 'lib/rubikon/errors.rb', line 89

def initialize(arg)
  super "Unexpected argument: #{arg}"
end