Exception: Rubikon::UnexpectedArgumentError
- Defined in:
- lib/rubikon/errors.rb
Overview
Raised if an argument is passed, that does not match a validation rule
Instance Method Summary collapse
-
#initialize(arg) ⇒ UnexpectedArgumentError
constructor
Creates a new error and stores the given argument value.
Constructor Details
#initialize(arg) ⇒ UnexpectedArgumentError
Creates a new error and stores the given argument value
89 90 91 |
# File 'lib/rubikon/errors.rb', line 89 def initialize(arg) super "Unexpected argument: #{arg}" end |